const path = require('path')
const yaml = require('js-yaml')
const fs = require('fs')
const CommonsChunkPlugin = require('webpack/lib/optimize/CommonsChunkPlugin')

/**
 *
 */
module.exports = function(customConfig) {
  const {appDir, nodeModulesDir} = customConfig
  const webpackDir = path.join(appDir, 'webpack')

  const webpackConfig = yaml.load(fs.readFileSync(`${appDir}/config/webpack.yml`, 'utf8'))
  const entryMap = createEntryMap(webpackDir)
  const commonsChunkPluginArr = createCommonsChunkPluginArr(webpackConfig)

  return {
    context: webpackDir,
    entry: entryMap,
    output: {
      path         : '/tmp/perch-app',
      publicPath   : '/',
      filename     : 'js/[name].js',
      chunkFilename: 'js/chunk/[id].js',
    },
    resolve: {
      modules: [
        webpackDir,
        nodeModulesDir,
      ],
      descriptionFiles: ['package.json'],
      mainFields: ['main', 'browser'],
      mainFiles: ['index'],
      extensions: [
        '.js',
      ],
      enforceExtension: false,
      enforceModuleExtension: false,
    },
    plugins: [
      ...commonsChunkPluginArr,
    ],
    devServer: {
      contentBase: webpackDir,
      host: '0.0.0.0',
      disableHostCheck: true,
      stats: {
        assets: false,
        colors: true,
        children: false,
        chunks: false,
        modules: false,
      },
    },
  }
}

/**
 *
 */
function createCommonsChunkPluginArr(webpackConfig) {
  const commonsChunkPluginArr = []

  for (const commonName in webpackConfig.commons) {
    const entryArrPath = webpackConfig.commons[commonName].map(function(entryName) {
      return `entries/${entryName}`
    })
    commonsChunkPluginArr.push(new CommonsChunkPlugin({
      name    : `commons/${commonName}`,
      filename: 'js/[name].js',
      chunks  : entryArrPath,
    }))
  }

  return commonsChunkPluginArr
}

/**
 *
 */
function createEntryMap(webpackDir) {
  const entryMap = {}
  const commonsDir = `${webpackDir}/commons`
  const entriesDir = `${webpackDir}/entries`

  if (fs.existsSync(entriesDir)) {
    for (const entry of getNamesFromDirectory(entriesDir)) {
      entryMap[`entries/${entry}`] = `./entries/${entry}`
    }
  }
  if (fs.existsSync(commonsDir)) {
    for (const common of getNamesFromDirectory(commonsDir)) {
      entryMap[`commons/${common}`] = `./commons/${common}`
    }
  }

  return entryMap
}

/**
 *
 */
function getNamesFromDirectory(filepath) {
  return fs
    .readdirSync(filepath)
    .filter(filename => filename[0] !== '#')
    .map(filename => {
      let entryName, ext
      if (fs.lstatSync(`${filepath}/${filename}`).isDirectory()) {
        entryName = filename
      } else {
        ext = path.extname(filename)
        entryName = filename.substr(0, filename.length - ext.length)
      }
      return entryName
    })
}